/*
 * Component: utils
 *
 * Description:
 *    Header file for helper functions related to hashing
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __IM_HASH_H_INCLUDED__
#define __IM_HASH_H_INCLUDED__

#include <unistd.h>
#include <sys/file.h>

#define DEBUGHASH(...) if (isDEBUGHASH) {openlog("imhash", LOG_PID, LOG_LOCAL3); syslog(LOG_DEBUG, __VA_ARGS__); closelog(); }

#define HASHDEPTH 3

/**
 * Checks whether a file/directory exists
 *
 * @param path   path to the file/directory to check
 */
bool exists( const char * path );

/**
 * Checks for the existence of the given pathname and creates any non-existing directories on it's way
 *
 * @param	dirName	name of the directory that should be created
 * @param mode		file creation mode, see `man mkdir(2)` defaults to "drwxr-xr-x"
 * @return 0 if succeeded, -1 if IOerror occurred
 */
int mkdirs( const char* dirName, unsigned long mode = S_IWUSR | S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );

/**
 * Returns the base dir of a file/directory
 * @param	directoryName	name of the file/directory who's base should be determined
 * 				shouldn't end with '/'
 * @return the base dir or NULL
 */
char* baseDir( const char* dirName );

/**
 * Returns the parent of a directory
 * @param	directoryName	name of the directory who's parent should be determined
 * @return the parent of directoryName
 */
char* parentDir( const char* directoryName );

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Creates a hashed Directory and a link in an old-style directory if needed
 * 
 * @param legacyDir		the old style directory that should contain a link to the newly created dir (or don't create a link if this parameter is NULL)
 * @param currentDir	where to put the new Directory
 * @param subDir			the subdirectory to be created.
 * all directories must end with a '/' character, if they don't, they will be truncated
 * if the subDir is "xxx/yyy/" and the currentDir is "/aaa/bbb/" the created directory will look
 * like "/aaa/bbb/x1/x2/x3/xxx/y1/y2/y3/yyy/" where x1, x2 and x3 are hash values generated from "xxx"
 * if legacyDir is present ("/tytyty/gygygy/"), after the process there will be a 
 * "/tytyty/gygygy/xxx/yyy/" which is a link pointing to the newly created directory
 * @return	0 if no error occured
 */
int createHashDirectory( const char* legacyDir, const char* currentDir, const char* subDir );

/*
 * removes a directory and it's link if needed
 * parametering is the same as createHashDirectory's
 */
int removeHashDirectory( const char* legacyDir, const char* currentDir, const char* subDir );

/**
 * Converts a path into it's hashed form.

 * Paths ending with '/' shall be treated as directories and each segment of the path shall be hashed.
 *
 * ("xxx/yyy/" shall be converted to "x1/x2/x3/xxx/y1/y2/y3/yyy/").
 *
 * Paths not ending with '/' shall be converted depending on the value of the <code>hashFile</code> parameter.
 * (ie. whether ending file name shall be hashed or not)
 *
 * If <code>hashFile</code> is <code>true</code>, then each segment of the path shall be hashed.
 * ("xxx/yyy" shall be converted to "x1/x2/x3/xxx/y1/y2/y3/yyy")
 *
 * If <code>hashFile</code> is <code>false</code>, then all but the last segment of the path shall be hashed.
 * ("xxx/yyy" shall be converted to "x1/x2/x3/xxx/yyy")
 *
 * the returned value should be released by the caller
 *
 * @param path	    the path to hash
 * @param hashFile true indicates to also hash trailing file name (if any)
 * @return the hashed path
 */
char * convertToHashed( const char * path, bool hashFile = false );


/**
 * Creates a hash directory for a file.
 * Generates a hash name for a filename and creates hash directories to place the file in.
 * If path to the file contains directory (or directories), the directories will also be hashed.
 *
 * Eg. if '<code>filePath</code>' is xxx/yyy/zzz, then the hash directory to create shall be 'x1/x2/x3/xxx/y1/y2/y3/yyy/z1/z2/z3/'
 *
 * @param baseDirPath path to the base directory, <code>filePath</code> shall be located in this directory
 * @param filePath path to the file to hash
 *
 * @return 0 if the hash directory could be successfully created, or non-zero if an error occurred
 */
int createHashDirectoryForFile( const char * baseDirPath, const char * filePath );


/**
 * Removes hash directory created for a file.
 * Generates a hash name for a filename and deletes empty hash directories on the path.
 * If path to the file contains directory (or directories), then directory names will also be hashed.
 *
 * Eg. if '<code>filePath</code>' is xxx/yyy/zzz, then the hash directory to delete shall be 'x1/x2/x3/xxx/y1/y2/y3/yyy/z1/z2/z3/'
 *
 * @param baseDirPath  path to the base directory, <code>filePath</code> shall be located in this directory
 * @param filePath path to the file to hash
 *
 * @return 0 if empty hash directories could be successfully deleted, or non-zero if an error occurred
 */
int removeHashDirectoryForFile( const char * baseDirPath, const char * filePath );


/*
 * we can release strings allocated by new from C
 */
void release( char* ptr );

#ifdef __cplusplus
}
#endif

#endif // __IM_HASH_H_INCLUDED__
